
# A common question is "How big should my sample be?". There's no simple answer,
#   but simulations can help you to decide.

# Earlier in the workshop, we simulated data for squirrel weights with sample size n = 10.
# What would happen if we used a smaller sample, or a bigger sample?
#   How good would our estimates be?

# The biological model
# ====================
# We will assume that squirrel weights are normally distributed, with mean
mu <- 1000  # grams, and standard deviation
sigma <- 150

# The observation model
# =====================

# Generate 1 sample of n=10 and check mean and SD

# We will assume that our sampling method gives a representative, random sample from
#   the population of squirrels. Then we can use 'rnorm()' to generate samples.
n <- 10  # sample size, we'll try different values later
( x <- rnorm(n, mu, sigma) )  # Produces a different sample each time we run it.
mean(x)

# Generating many samples and many estimates
# ==========================================

## Section A
## Generate 500 samples with n=10

# We saw with the plastic chips that each person got a different estimate of the population
#   mean. Let's do 500 samples and look at the distribution of the estimates.
# For that we need a loop in R.

iter <- 500  # The number of iterations = number of samples to generate
sampleSize10 <- numeric(iter)  # Create a store to hold all the estimates with sample size 10.

for(i in 1:iter)  {
  x <- rnorm(10, mu, sigma)
  sampleSize10[i] <- mean(x)
}

head(sampleSize10)  # Look at the first 6 values
hist(sampleSize10)  # Display all 500

# Check the mean of the estimates
mean(sampleSize10)
  # if our method is unbiased, this should be close to the true value.

# Try different sample sizes
# ==========================

## Section B
## Generate 500 samples with n=4.
# Put the result in an object called 'sampleSize4' and check for bias.
# HINT: Copy-paste the code from section A and then change the copy.


## Section C
## Generate 500 samples with n=30.
# Put the result in an object called 'sampleSize30' and check for bias.



# Compare the estimates
# =====================

# Method 1: look at histograms
# ----------------------------
par(mfrow = c(3, 1))  # Put 3 histograms into the same plot
hist(sampleSize10)
##### Add the histograms for sampleSize4 and sampleSize30.

# Which histogram is narrowest?
# ...not easy to see when the x axes are different.

# For comparison, give them all the same scale on the x axis (and proper titles)
#  and add the true value
hist(sampleSize10, xlim=c(700, 1300), main="n = 10")
abline(v = mu, col='red')
##### Add the histograms and red lines for sampleSize4 and sampleSize30.

par(mfrow = c(1, 1))  # Change back to a single plot when we have finished.

# Method 2: Calculate errors
# --------------------------
# Since we know the true value, we can calculate the errors, and summarise these as
#   Root Mean Square Error (RMSE)
sqrt(mean((sampleSize10 - mu)^2))
##### Do the same for sampleSize4 and sampleSize30


# Which has the lowest RMSE?

# Method 3: Beeswarm plots
# ------------------------
library(beeswarm)
# The beeswarm function needs a data frame as its input
df <- data.frame(n10 = sampleSize10)
##### df <- data.frame(n4 = sampleSize4, n10 = sampleSize10, n30 = sampleSize30)
head(df)

beeswarm(df)
# Ok... now let's make that prettier
beeswarm(df, method='hex', cex=0.5, col='blue', las=1,
  xlab="Sample size", ylab="Estimated mean weight of squirrels")
# Add a horizontal line with the true value:
abline(h = mu, col='red', lwd=2)

# Which sample size is best? Why?

# Decisions, decisions!
# =====================
# Bigger samples give more accurate estimates, but are also more expensive.
# How accurate do you need to be? How much money can you spend?

# Suppose we wanted to be within 100g of the true value, ie, between 900 and 1100g
# Add these to the beeswarm plot:
abline(h = c(mu - 100, mu + 100), col='red', lty=2)

# What's the probability that we will get an estimate with an error greater than 100g?
mean(abs(sampleSize10 - mu) > 100)
# Do the same for n=4 and n=30


# How robust is your decision?
# ============================
# Try running the code above with different values of mu and sigma:
#  a. How would your decision be affected if mu = 1200g instead of 1000?
#  b. How would your decision be affected if sigma = 75g instead of 150?
#  c. How would your decision be affected if sigma = 300g instead of 150?
