
# BAYESIAN ANALYSIS OF BELUGA WHALE SIGHTINGS IN COOKE INLET


# This script is based on the analysis presented in Wade, P. R. (2001)
#   The conservation of exploited species in an uncertain world: novel
#   methods and the failure of traditional techniques. Ch 6 in Reynolds,
#   Mace, Redford and Robinson (eds) Conservation of exploited species.
#   Cambridge University Press, Cambridge UK, pp 120-124 and 132-134.

## ANALYSIS OF THE DATA FOR 1994 TO 1998
## =====================================

# Read in the data:
# '''''''''''''''''

# These are the number of sightings of beluga whales during annual aerial
#   surveys of Cooke Inlet for the years 1994 to 1998:
sightings98 <- c(281,324,307,264,193)
Year98 <- 1994:1998

# Convert to log, so that the trend can be expressed as a percentage change
#   per year:
logs98 <- log(sightings98)

# Plot the data:
# ''''''''''''''
plot(Year98, logs98)

## Frequentist analysis
## --------------------
# Use 'lm' to fit a regression line:
lm98 <- lm(logs98 ~ Year98)
# ...and add it to the graph:
abline(lm98, lty=2, col='red')

# Look at the slope (the 'Year98' line):
summary(lm98)
  # The estimated slope is -0.096 (or -9.6%) per year with a SE of 0.0494.
# Look at the confidence interval for 'Year98':
confint(lm98, 'Year98')
  # It goes from -25% to +6%, so no evidence of a decline.

## Bayesian analysis
## -----------------

# With a completely uninformative prior, the posterior probabilities are equal
#   to the frequentist likelihood, suitably scaled.
# Here, we can use the same t-distribution that was used to calculate the
#   confidence interval from the regression, ie. mean = -0.09561, SE = 0.04937
#   and 3 degrees of freedom. (Note that with this regression model the
#   degrees of freedom = n - 2, as we estimate 2 parameters from the data.)
# For this we need the dt2 function in package wiqid version 0.1.0 or later

library(wiqid)

# Plot the posterior distribution:
# '''''''''''''''''''''''''''''''
# We'll plot it for trends between -30% and +10%:
trend <- seq(-0.3, 0.1, length=101)
prob <- dt2(trend, -0.09561, 0.04937, 3)
plot(trend, prob, type='l', ylab="Posterior probability density",
  xlab="Population trend", las=1)

# Add vertical lines for trend = 0 and trend = -0.05:
abline(v = c(0, -0.05), lty=3, col='red')
abline(h=0, col='grey')

# Calculate the probabilities of trend > 0 and trend < -0.05:
# '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

# We are interested in the area to the right of trend=0, so we put
#   'lower.tail = FALSE':
( prob1 <- pt2(0, -0.09561, 0.04937, 3, lower.tail=FALSE) )
# 0.074, so the probability that the trend is positive is only 7.4%.

# Now we are interested in the area to the left of trend = -0.05, so we put
#   'lower.tail = TRUE':
( prob2 <- pt2(-0.05, -0.09561, 0.04937, 3, lower.tail=TRUE) )
  # 0.788, so the probability that the decline is greater than 5% per year
  #   is 79% !
( prob3 <- 1 - prob1 - prob2)
# Add these to the graph:
txt <- paste0(round(c(prob1, prob3, prob2)*100), "%")
text(c(0.025, -0.025, -0.1), c(0.025, 0.05, 0.15), txt, col='red')


## If you want to do the analysis for the data from 1994 to 2000, just change
##   the values in the script as appropriate. The sightings for 1999 and 2000 were 217 and 184 respectively.


## DECISION MAKING
## ===============

# Loss functions
# --------------

# This is the matrix of losses:
loss <- matrix(c(0,1,2,1,0,1,2,1,0), 3,3)
rownames(loss) <- c("Panic","Act","Relax")
colnames(loss) <- c("Steep decline","Small decline","No decline")
loss

# Calculating expected loss
# -------------------------

# For the data for 1994 to 1998, the respective posterior probabilities are:
post.probs98 <- c(0.79, 0.14, 0.07)
sum(post.probs98)
  # They add up to 1.

# We can use matrix algebra to do the multiplication and addition in one
#   step:
loss %*% post.probs98

# The smallest expected loss is that due to the 'Panic' decision, so this
#   was the most appropriate course of action.

## You can put in the posterior probabilities for the 1994-1999 and 1994-2000
##   data sets and calculate the expected losses and see the best course
##   of action for each year.

## Doing a prettier plot
## =====================
# The basic graphic is okay, but we can make it prettier, and label the y-axis
#   with the actual numbers of sightings instead of the logs:
plot(Year98, logs98,  yaxt='n', pch=16,
    xlab="Year", ylab="Sightings (log scale)")
# yaxt='n' told R not to draw the y axis; we do that now, with tick marks
#   for 200, 250 and 300:
ticks <- c(200, 250, 300)
axis(2, log(ticks), ticks, las=1)

# Use ?par for help on yaxt and las.


